﻿using System;
using System.Linq;
using System.Text.RegularExpressions;
using Telerik.Windows.Documents.Spreadsheet.Model;
using Telerik.Windows.Documents.Spreadsheet.Utilities;

namespace My.ExtensionMethods
{
    public static class SpreadsheetExtensions
    {
        public static int ExcelColumnNameToNumber(string columnName)
        {
            if (string.IsNullOrEmpty(columnName)) throw new ArgumentNullException("columnName");

            columnName = columnName.ToUpperInvariant();

            int sum = 0;

            for (int i = 0; i < columnName.Length; i++)
            {
                sum *= 26;
                sum += (columnName[i] - 'A' + 1);
            }

            return sum;
        }

        public static CellSelection Range(this Worksheet sheet, string range)
        {
            string patternRC = @"^r(?<ulRow>\d+)c(?<ulCol>\d+)(:r(?<lrRow>\d+)c(?<lrCol>\d+))?$";
            string pattern = @"^(?<ulCol>[a-z]+)(?<ulRow>\d+)(:(?<lrCol>[a-z]+)(?<lrRow>\d+))?$";

            Match matchRC = Regex.Match(range, patternRC, RegexOptions.IgnoreCase);
            Match match = Regex.Match(range, pattern, RegexOptions.IgnoreCase);

            string ulCol, ulRow, lrCol, lrRow;
            int ulCol0, ulRow0, lrCol0, lrRow0;

            if (matchRC.Success)
            {
                ulRow = matchRC.Groups["ulRow"].Value;
                ulCol = matchRC.Groups["ulCol"].Value;
                lrRow = matchRC.Groups["lrRow"].Value;
                lrCol = matchRC.Groups["lrCol"].Value;

                if (lrRow.Length == 0) lrRow = ulRow;
                if (lrCol.Length == 0) lrCol = ulCol;

                ulCol0 = Convert.ToInt32(ulCol) - 1;
                ulRow0 = Convert.ToInt32(ulRow) - 1;
                lrCol0 = Convert.ToInt32(lrCol) - 1;
                lrRow0 = Convert.ToInt32(lrRow) - 1;
            }
            else if (match.Success)
            {
                ulCol = match.Groups["ulCol"].Value;
                ulRow = match.Groups["ulRow"].Value;
                lrCol = match.Groups["lrCol"].Value;
                lrRow = match.Groups["lrRow"].Value;

                if (lrRow.Length == 0) lrRow = ulRow;
                if (lrCol.Length == 0) lrCol = ulCol;

                ulCol0 = ExcelColumnNameToNumber(ulCol) - 1;
                ulRow0 = Convert.ToInt32(ulRow) - 1;
                lrCol0 = ExcelColumnNameToNumber(lrCol) - 1;
                lrRow0 = Convert.ToInt32(lrRow) - 1;
            }
            else
                throw new Exception("Invalid Excel cell address: " + range);

            return sheet.Cells[ulRow0, ulCol0, lrRow0, lrCol0];
        }

        public static CellSelection _Merge(this CellSelection cells)
        {
            cells.Merge();
            return cells;
        }

        public static CellSelection _SetValue(this CellSelection cells, int value)
        {
            cells.SetValue(value);
            return cells;
        }

        public static CellSelection _Set_R1C1_ValueAsText(this CellSelection cells, string value)
        {
            cells.Worksheet.Cells[
                cells.CellRanges.First().GetFirstRow().GetFirstColumn()
            ]
            .SetValueAsText(value);

            return cells;
        }

        public static CellSelection _SetDateTimeValue(this CellSelection cells, DateTime? value)
        {
            if (value != null) cells.SetValue(value.Value);
            return cells;
        }

        public static CellSelection _Bold(this CellSelection cells)
        {
            cells.SetIsBold(true);
            return cells;
        }

        public static CellSelection _Italic(this CellSelection cells)
        {
            cells.SetIsItalic(true);
            return cells;
        }

        public static CellSelection _Center(this CellSelection cells)
        {
            cells.SetHorizontalAlignment(RadHorizontalAlignment.Center);
            return cells;
        }

        public static CellSelection _Left(this CellSelection cells)
        {
            cells.SetHorizontalAlignment(RadHorizontalAlignment.Left);
            return cells;
        }

        public static CellSelection _Middle(this CellSelection cells)
        {
            cells.SetVerticalAlignment(RadVerticalAlignment.Center);
            return cells;
        }

        public static CellSelection _FieldLabel(this CellSelection cells, string label)
        {
            cells.Merge();

            cells.SetValueAsText(label);
            cells.SetFontFamily(new ThemableFontFamily("Times New Roman"));
            cells.SetFontSize(UnitHelper.PointToDip(10));
            cells.SetIsItalic(true);
            cells.SetHorizontalAlignment(RadHorizontalAlignment.Right);

            return cells;
        }

        public static CellSelection _FieldText(this CellSelection cells, string value)
        {
            cells.Merge();

            cells
                ._Set_R1C1_ValueAsText(value)
                ._Center()
                ._Box()
                ._Font("Times New Roman", 10);

            return cells;
        }

        public static CellSelection _FieldValue(this CellSelection cells, int value)
        {
            cells.Merge();

            cells.Worksheet.Cells[
                cells.CellRanges.First().GetFirstRow().GetFirstColumn()
            ]
            ._SetValue(value);

            cells
                ._Center()
                ._Box()
                ._Font("Times New Roman", 10);

            return cells;
        }

        public static CellSelection _FieldDate(this CellSelection cells, DateTime? value)
        {
            cells.Merge();

            cells.Worksheet.Cells[
                cells.CellRanges.First().GetFirstRow().GetFirstColumn()
            ]
            ._SetDateTimeValue(value);

            cells
                ._Center()
                ._Box()
                ._Font("Times New Roman", 10);

            return cells;
        }

        public static CellSelection _Background(this CellSelection cells, System.Windows.Media.Color value)
        {
            cells.SetFill(new PatternFill
                ( patternType: PatternType.Solid
                , patternColor: value
                , backgroundColor: System.Windows.Media.Colors.White // doesn't matter for Solid
                )
            );

            return cells;
        }

        public static CellSelection _Box(this CellSelection cells)
        {
            // https://docs.telerik.com/devtools/document-processing/libraries/radspreadprocessing/working-with-cells/get-set-clear-properties
            CellBorder edge = new CellBorder(CellBorderStyle.Thin, ThemableColor.FromArgb(alfa: 1, red: 0, green: 0, blue: 0));
            cells.SetBorders(new CellBorders(edge));            
            return cells;
        }

        public static CellSelection _TopBot(this CellSelection cells)
        {
            CellBorder edge = new CellBorder(CellBorderStyle.Thin, ThemableColor.FromArgb(alfa: 1, red: 0, green: 0, blue: 0));
            cells.SetBorders(new CellBorders(null, edge, null, edge, null, null, null, null));
            return cells;
        }

        public static CellSelection _Font(this CellSelection cells, string family, int points)
        {
            cells.SetFontFamily(new ThemableFontFamily(family));
            cells.SetFontSize(UnitHelper.PointToDip(points));
            return cells;
        }

        public static CellSelection _FontSize(this CellSelection cells, int points)
        {
            cells.SetFontSize(UnitHelper.PointToDip(points));
            return cells;
        }

        public static CellSelection _FontFamily(this CellSelection cells, string family)
        {
            cells.SetFontFamily(new ThemableFontFamily(family));
            return cells;
        }
    }
}